#include "src/frontend/parser.h"
#include "src/frontend/tokenize.h"
#include "src/util/config.h"
#include "src/backend/CodeGen.h"
#include <iostream>
#include <istream>
#include <memory>
#include <ostream>
#include <sstream>
#include <string>

int main(int argc, char **argv) {
    compiler::config::parse_args(argc, argv);

    std::unique_ptr<std::istream> input_stream(std::move(compiler::config::input));
    if (!input_stream) {
        std::cerr << "Error: Input file is not specified or cannot be opened." << std::endl;
        return 1;
    }

    // Tokenizer部分
    auto &singleton = compiler::frontend::Tokenize::singleton();
    singleton.setup(input_stream);
    auto [tokens, result] = singleton.input_file_to_json();
    if (!result) {
        std::cerr << "Error happened at tokenizer." << std::endl;
        return 1;
    }
    if (compiler::config::print_token) {
        auto token = singleton.dump_tokens();
        *compiler::config::output << token;
    }

    // Parser部分
    compiler::frontend::Parser parser(tokens);
    auto nodes = parser.parse();
    if (compiler::config::print_ast) {
        auto parse_json = parser.jsonify(nodes);
        *compiler::config::output << parse_json;
    }

    if (compiler::config::generate_assembly) {
        std::ostringstream os;
        if (!arm_gen_head(nodes, os)) {
            std::cerr << last_arm_gen_erro << std::endl;
            return 1;
        }
        *compiler::config::output << os.str();
        std::cout << "Assembly code generated successfully." << std::endl;
    } else {
        std::cout << "Compilation completed." << std::endl;
    }

    return 0;
}