#pragma once
#include <sstream>
#include <algorithm>
#include <iostream>
#include <string>
#include <vector>
#include <memory>
#include <functional>
// #include "SSA.cpp"
#include "../util/node.h"
#include "../util/type.h"

using namespace compiler::node;
namespace type = compiler::type;

extern char const *last_arm_gen_erro;

int reg_allocate(int reg, std::ostringstream &os) ;

int reg_allocate(std::ostringstream &os) ;

void reg_deallocate(int reg, std::ostringstream &os) ;

int is_valid_immediate_value(int _value, int &new_value) ;
inline bool is_valid_immediate_value(int _value) ;

bool get_target_var_offset(std::shared_ptr<VarTypeNode> const &target_var, int &offset) ;

template<class NODE_TYPE>
bool arm_gen_array(std::vector<std::shared_ptr<NODE_TYPE>> &nodes, std::ostringstream &oss) ;

template<typename OPERATE>
inline long long calculation(long long operand0, long long operand1, OPERATE operate) ;

template<typename OPERATE>
bool calculation(const std::vector<std::shared_ptr<Node>>& parameters, const std::shared_ptr<Node>& parent, OPERATE operate, std::shared_ptr<LiteralNode>& literal) ;


std::function<long long(long long, long long)> get_operation_function(type::OPERATION_TYPE op_type) ;

bool check_literal(std::shared_ptr<Node> const &node, std::shared_ptr<LiteralNode> &literal) ;

bool array_gen(
    const std::vector<int> &dimensions,
    const std::shared_ptr<ArrayNode> &init_value,
    std::vector<long long> &_init_value,
    std::vector<int> &init_array_index,
    int dimensions_index
) ;

bool array_gen(std::vector<int> const &dimensions, std::shared_ptr<ArrayNode> const &init_value, std::vector<std::shared_ptr<Node>> &_init_value,
		std::vector<int> &init_array_index, int dimensions_index) ;

bool array_gen(std::vector<int> const &dimensions, std::shared_ptr<ArrayNode> &init_value, std::vector<long long> &_init_value) ;

bool array_gen(std::vector<int> const &dimensions, std::shared_ptr<ArrayNode> &init_value, std::vector<std::shared_ptr<Node>> &_init_value) ;

bool calculate_block_var_offset(std::shared_ptr<BlockNode> &block, int &local_var_total_size) ;

bool process_statement(const std::shared_ptr<Node> &stmt, int &local_var_total_size) ;

bool calculate_funtion_var_offset(std::shared_ptr<FuncDefNode> &fun_def, int &local_var_total_size) ;

std::shared_ptr<Node> get_var_type_init_value(std::shared_ptr<VarTypeNode> const &type) ;

bool literal_vector2int_vector(std::vector<std::shared_ptr<Node>> const &literal_vector, std::vector<int> &int_vector) ;

bool get_array_access_index_expression(std::shared_ptr<VarTypeNode> const &target_var, std::vector<std::shared_ptr<Node>> const &indexes,
    std::shared_ptr<Node> &expression, bool &partial_access);

bool process_var_def(std::shared_ptr<Node> const &node, std::ostringstream &os) ;

bool process_global_var(const std::shared_ptr<VarTypeNode> &type, const std::shared_ptr<Node> &init_value, std::ostringstream &os) ;

bool process_local_var(std::shared_ptr<VarFuncNode> const &node, std::ostringstream &os) ;

bool process_local_var_item(const std::shared_ptr<VarTypeNode> &type, const std::shared_ptr<Node> &_init_value, std::ostringstream &os) ;

bool process_func_def(std::shared_ptr<Node> const &node, std::ostringstream &os) ;

bool process_func_prologue(int local_var_total_size, std::ostringstream &os) ;

bool process_func_parameters(const std::vector<std::shared_ptr<VarFuncNode>> &parameters, std::ostringstream &os) ;

bool process_var(std::shared_ptr<Node> const &node, std::ostringstream &os) ;

bool process_func(std::shared_ptr<Node> const &node, std::ostringstream &os) ;

bool process_array_index(std::shared_ptr<Node> const &node, std::ostringstream &os) ;

bool arm_gen(std::shared_ptr<Node> const &node, std::ostringstream &os);

bool arm_gen_head(std::shared_ptr<Node> const &node, std::ostringstream &os) ;
