#include "tokenize.h"
#include "../util/type.h"
#include "fmt/core.h"
#include "fmt/format.h"
#include "tokens.h"
#include <cctype>
#include <iterator>
#include <memory>
#include <string>
#include <tuple>
#include <vector>
namespace compiler::frontend {
auto Tokenize::input_file_to_json()
    -> std::tuple<std::vector<std::shared_ptr<Token>>, bool> {
  const char *current = content.c_str();
  while (*current) {
    if (isspace(*current)) {
      ++current;
      continue;
    }
    // 单行注释
    if (is_start_with(current, type::sysy_comment.at(0).data())) {
      current += 2;
      while ('\n' != *(current++)) {
        ;
      }
      continue;
    }
    // 多行注释
    if (is_start_with(current, type::sysy_comment.at(1).data())) {
      while (!is_start_with(++current, type::sysy_comment.at(2).data())) {
        ;
      }
      current += 2;
      continue;
    }

    if (is_identifier_start_letter(*current)) {
      tokens.push_back(new_identifier_token(current));
      continue;
    }
    if (isdigit(*current)) {
      tokens.push_back(new_literal_token(current));
      continue;
    }
    if (is_multi_letter_symbol(current)) {
      tokens.push_back(new_multi_letter_symbol(current));
      continue;
    }
    if (is_single_letter_symbol(current)) {
      tokens.push_back(new_single_letter_symbol(current));
      continue;
    }
    // TODO: 这个地方可以加如报错位置，需要添加token在代码中的位置。
    last_tokenize_error_msg = "Unexpected token!";
    return std::make_tuple(tokens, false);
  }
  auto flag = set_token_type(tokens);
  return std::make_tuple(tokens, flag);
}
auto Tokenize::dump_tokens() -> std::string {
  fmt::memory_buffer buffer;
  fmt::format_to(std::back_inserter(buffer), "[{}]", fmt::join(tokens, ","));
  return fmt::to_string(buffer);
}
} // namespace compiler::frontend
