#pragma once
#include "../util/type.h"
#include "fmt/core.h"
#include "fmt/format.h"
#include <cstddef>
#include <memory>
#include <string>

namespace compiler::frontend {
namespace type = compiler::type;
class Token {
  char const *start;
  std::size_t length;
  std::string name;
  type::TOKEN_TYPE type;

public:
  Token()
      : start(nullptr), length(-1), name(),
        type(type::TOKEN_TYPE::T_INVALID) {};
  Token(const Token &) = default;
  Token(Token &&) = default;
  auto operator=(const Token &) -> Token & = default;
  auto operator=(Token &&) -> Token & = default;
  Token(const char *start, std::size_t length, const char *const name,
        type::TOKEN_TYPE type)
      : start(start), length(length), name(name), type(type) {};
  Token(char const *start, std::size_t length, char const *const name)
      : Token(start, length, name, type::TOKEN_TYPE::T_INVALID) {};
  ~Token() = default;
  [[nodiscard]] auto get_length() const -> std::size_t;
  void set_length(int len);
  [[nodiscard]] auto get_name() const -> const std::string &;
  void set_name(const std::string &name);
  [[nodiscard]] auto get_start() const -> const char *;
  void set_start(const char *start);
  [[nodiscard]] auto get_type() const -> type::TOKEN_TYPE;
  void set_type(type::TOKEN_TYPE type);
};

} // namespace compiler::frontend
// template <>
// struct fmt::formatter<compiler::frontend::Token>
//     : fmt::formatter<std::string_view> {
//   constexpr auto parse(fmt::format_parse_context &ctx) { return ctx.begin();
//   } auto format(const compiler::frontend::Token &token,
//               fmt::format_context &ctx) const {
//     return fmt::format_to(ctx.out(), R"({{"name": "{}", "type": "{}"}})",
//                           token.get_name(),
//                           compiler::type::token_type_to_str(token.get_type()));
//   }
// };
template <>
struct fmt::formatter<std::shared_ptr<compiler::frontend::Token>>
    : fmt::formatter<std::string_view> {
  constexpr auto parse(fmt::format_parse_context &ctx) { return ctx.begin(); }
  auto format(const std::shared_ptr<compiler::frontend::Token> &token,
              fmt::format_context &ctx) const {
    return fmt::format_to(
        ctx.out(), R"({{"name": "{}", "type": "{}"}})", token->get_name(),
        compiler::type::token_type_to_string(token->get_type()));
  }
};
