#include "config.h"
#include "cxxopts.hpp"
#include <cstdio>
#include <fstream>
#include <iostream>
#include <istream>
#include <ostream>
#include <string>

namespace compiler::config {
    using std::ofstream;

    struct NoOpDeleter {
        void operator()(std::ostream *) const {}
    };

    int optimize_level = 0;
    std::unique_ptr<std::ifstream> input = nullptr;
    std::unique_ptr<std::ofstream> output = nullptr;
    bool debug = false;
    bool print_ast = false;
    bool print_log = false;
    bool print_token = false;
    bool generate_assembly = false;
    std::string input_filename = "<stdin>";

    void parse_args(int argc, char **argv) {
        cxxopts::Options options("compiler", "Still in development, now testing frontend.");
        options.add_options()
            ("d,debug", "debug mode")
            ("O,optimization", "Optimization level", cxxopts::value<int>()->default_value("0"))
            ("o,output", "Output file", cxxopts::value<std::string>()->default_value("output.txt"))
            ("v,version", "1.0")
            ("print_token", "Print Tokens")
            ("print_ast", "Print AST")
            ("print_log", "Print Log")
            ("h,help", "Print help")
            ("S,assembly", "Generate assembly file");

        auto result = options.parse(argc, argv);

        if (result.count("debug")) {
            debug = true;
        }
        if (result["print_ast"].as<bool>()) {
            print_ast = true;
        }
        if (result["print_token"].as<bool>()) {
            print_token = true;
        }
        if (result.count("print_log")) {
            print_log = true;
        }
        if (result.count("version")) {
            std::cout << "v0.1";
            exit(0);
        }
        if (result.count("optimization")) {
            optimize_level = result["optimization"].as<int>();
        }
        if (result.count("output")) {
            auto output_file = result["output"].as<std::string>();
            if (output_file != "-") {
                output = std::make_unique<std::ofstream>(output_file, std::ofstream::out);
            }
        }
        if (result.count("assembly")) {
            generate_assembly = true;
        }
        if (result.count("help")) {
            std::cout << options.help() << std::endl;
            exit(0);
        }

        auto positional_args = result.unmatched();
        if (!positional_args.empty()) {
            input_filename = positional_args[0];
            if (input_filename != "-") {
                input = std::make_unique<std::ifstream>(input_filename);
            }
        }
    }
}
